; gpio.s
; Desenvolvido para a placa EK-TM4C1294XL
; Prof. Guilherme de S. Peron	- 12/03/2018
; Prof. Marcos E. P. Monteiro	- 12/03/2018
; Prof. DaLuz           		- 25/02/2022

;################################################################################
        THUMB                        ; Instrues do tipo Thumb-2
;################################################################################
; Definies dos Registradores Gerais.  Obs: *(EQU)=(EQUATE)*
SYSCTL_RCGCGPIO_R	 		EQU		0x400FE608
SYSCTL_PRGPIO_R		 		EQU		0x400FEA08
; Definies dos Registradores especiais do PORT K
GPIO_PORTK_LOCK_R    		EQU		0x40061520
GPIO_PORTK_CR_R      		EQU		0x40061524
GPIO_PORTK_AMSEL_R      	EQU		0x40061528
GPIO_PORTK_PCTL_R       	EQU		0x4006152C
GPIO_PORTK_DIR_R     		EQU		0x40061400
GPIO_PORTK_AFSEL_R   		EQU		0x40061420
GPIO_PORTK_DEN_R     		EQU		0x4006151C
GPIO_PORTK_PUR_R     		EQU		0x40061510	
GPIO_PORTK_DATA_R    		EQU		0x400613FC
GPIO_PORTK               	EQU		2_000001000000000	;pg 383 datasheet
; Definies dos Registradores especiais do PORT N
GPIO_PORTN_AHB_LOCK_R    	EQU		0x40064520
GPIO_PORTN_AHB_CR_R      	EQU		0x40064524
GPIO_PORTN_AHB_AMSEL_R   	EQU		0x40064528
GPIO_PORTN_AHB_PCTL_R    	EQU		0x4006452C
GPIO_PORTN_AHB_DIR_R     	EQU		0x40064400
GPIO_PORTN_AHB_AFSEL_R   	EQU		0x40064420
GPIO_PORTN_AHB_DEN_R     	EQU		0x4006451C
GPIO_PORTN_AHB_PUR_R     	EQU		0x40064510	
GPIO_PORTN_AHB_DATA_R    	EQU		0x400643FC
GPIO_PORTN               	EQU		2_001000000000000	;pg 382 datasheet
; Definies dos Registradores especiais do PORT M
GPIO_PORTM_LOCK_R    		EQU		0x40063520
GPIO_PORTM_CR_R      		EQU		0x40063524
GPIO_PORTM_AMSEL_R      	EQU		0x40063528
GPIO_PORTM_PCTL_R       	EQU		0x4006352C
GPIO_PORTM_DIR_R     		EQU		0x40063400
GPIO_PORTM_AFSEL_R   		EQU		0x40063420
GPIO_PORTM_DEN_R     		EQU		0x4006351C
GPIO_PORTM_PUR_R     		EQU		0x40063510	
GPIO_PORTM_DATA_R    		EQU		0x400633FC
GPIO_PORTM_IM_R         	EQU		0x40063410
GPIO_PORTM_IS_R         	EQU		0x40063404
GPIO_PORTM_IBE_R        	EQU		0x40063408
GPIO_PORTM_IEV_R        	EQU		0x4006340C
GPIO_PORTM_ICR_R        	EQU		0x4006341C
GPIO_PORTM               	EQU		2_000100000000000	;pg 382 datasheet
;Outros Defines:
NVIC_PRI18_R         		EQU     0xE000E448
NVIC_EN2_R              	EQU		0xE000E108

;################################################################################
; rea de Cdigo - Tudo abaixo da diretiva a seguir ser armazenado na memria de 
;                  cdigo
	AREA    |.text|, CODE, READONLY, ALIGN=2
	; Se alguma funo do arquivo for chamada em outro arquivo	
	EXPORT GPIO_Init            ; Permite chamar GPIO_Init de outro arquivo
	EXPORT PortN_Output			; Permite chamar PortN_Output de outro arquivo
	EXPORT GPIOPortM_Handler	; Permite chamar a Interrupo de outro arquivo

;################################################################################
; Funo GPIO_Init
; Parmetro de entrada: No tem
; Parmetro de sada: No tem
;################################################################################
GPIO_Init
; 1. Ativar o clock para a porta setando o bit correspondente no registrador RCGCGPIO,
; aps isso verificar no PRGPIO se a porta est pronta para uso.
; enable clock to GPIOF at clock gating register
	LDR     R0, =SYSCTL_RCGCGPIO_R  			;Carrega o endereo do registrador RCGCGPIO
	MOV		R1, #GPIO_PORTN                 	;Seta o bit da porta N
	ORR     R1, #GPIO_PORTM						;Seta o bit da porta M, fazendo com OR
	ORR     R1, #GPIO_PORTK						;Seta o bit da porta K, fazendo com OR
    STR     R1, [R0]							;Move para a memria os bits das portas no endereo do RCGCGPIO
    LDR     R0, =SYSCTL_PRGPIO_R				;Carrega o endereo do PRGPIO para esperar os GPIO ficarem prontos
EsperaGPIO
	LDR     R1, [R0]							;L da memria o contedo do endereo do registrador
	MOV     R2, #GPIO_PORTN                 	;Seta os bits correspondentes s portas para fazer a comparao
	ORR     R2, #GPIO_PORTM                 	;Seta o bit da porta M, fazendo com OR
	ORR     R2, #GPIO_PORTK                 	;Seta o bit da porta M, fazendo com OR
    TST     R1, R2								;Testa o R1 com R2 fazendo R1 & R2
    BEQ     EsperaGPIO					    	;Se o flag Z=1, volta para o lao. Seno continua executando

; 2. Limpar o AMSEL para desabilitar a analgica
    MOV     R1, #0x00							;Colocar 0 no registrador para desabilitar a funo analgica
    LDR     R0, =GPIO_PORTK_AMSEL_R     		;Carrega o R0 com o endereo do AMSEL para a porta K
    STR     R1, [R0]							;Guarda no registrador AMSEL da porta K da memria
    LDR     R0, =GPIO_PORTM_AMSEL_R				;Carrega o R0 com o endereo do AMSEL para a porta M
    STR     R1, [R0]					    	;Guarda no registrador AMSEL da porta M da memria
    LDR     R0, =GPIO_PORTN_AHB_AMSEL_R			;Carrega o R0 com o endereo do AMSEL para a porta N
    STR     R1, [R0]					    	;Guarda no registrador AMSEL da porta N da memria

; 3. Limpar PCTL para selecionar o GPIO
    MOV     R1, #0x00					    	;Colocar 0 no registrador para selecionar o modo GPIO
    LDR     R0, =GPIO_PORTK_PCTL_R				;Carrega o R0 com o endereo do PCTL para a porta K
    STR     R1, [R0]                        	;Guarda no registrador PCTL da porta J da memria
    LDR     R0, =GPIO_PORTM_PCTL_R				;Carrega o R0 com o endereo do PCTL para a porta M
    STR     R1, [R0]                        	;Guarda no registrador PCTL da porta M da memria
    LDR     R0, =GPIO_PORTN_AHB_PCTL_R      	;Carrega o R0 com o endereo do PCTL para a porta N
    STR     R1, [R0]                        	;Guarda no registrador PCTL da porta N da memria

; 4. DIR para 0 se for entrada, 1 se for sada
    LDR     R0, =GPIO_PORTK_DIR_R				;Carrega o R0 com o endereo do DIR para a porta K
	MOV     R1, #2_10000000						;PN1
    STR     R1, [R0]							;Guarda no registrador
    LDR     R0, =GPIO_PORTN_AHB_DIR_R			;Carrega o R0 com o endereo do DIR para a porta N
	MOV     R1, #2_00000001						;PN1 & PN0 para LED
	ORR     R1, #2_00000010						;Enviar o valor 0x03 para habilitar os pinos como sada 
    STR     R1, [R0]							;Guarda no registrador
; O certo era verificar os outros bits da PJ para no transformar entradas em sadas desnecessrias
	LDR     R0, =GPIO_PORTM_DIR_R				;Carrega o R0 com o endereo do DIR para a porta M
	MOV     R1, #0x00							;Colocar 0 no registrador DIR para funcionar como entrada
    STR     R1, [R0]							;Guarda no registrador

; 5. Limpar os bits AFSEL para 0 para selecionar GPIO Sem funo alternativa
    MOV     R1, #0x00							;Colocar o valor 0 para no setar funo alternativa
    LDR     R0, =GPIO_PORTK_AFSEL_R				;Carrega o endereo do AFSEL da porta K
    STR     R1, [R0]							;Escreve na porta
    LDR     R0, =GPIO_PORTN_AHB_AFSEL_R			;Carrega o endereo do AFSEL da porta N
    STR     R1, [R0]							;Escreve na porta
    LDR     R0, =GPIO_PORTM_AFSEL_R				;Carrega o endereo do AFSEL da porta M
    STR     R1, [R0]							;Escreve na porta

; 6. Setar os bits de DEN para habilitar I/O digital
    LDR     R0, =GPIO_PORTK_DEN_R		    	;Carrega o endereo do DEN
    LDR     R1, [R0]					    	;Ler da Memria o registrador GPIO_PORTK_DEN_R
    MOV     R2, #2_10000000
    ORR		R1, R2
	STR     R1, [R0]							;Escreve no registrador da memria funcionalidade digital 
	
	LDR     R0, =GPIO_PORTN_AHB_DEN_R			;Carrega o endereo do DEN
    LDR     R1, [R0]					    	;Ler da Memria o registrador GPIO_PORTN_DEN_R
	MOV     R2, #2_00000001     
    ORR     R2, #2_00000010
    ORR		R1, R2
	STR     R1, [R0]                    		;Escreve no registrador da memria funcionalidade digital
	
	LDR     R0, =GPIO_PORTM_DEN_R				;Carrega o endereo do DEN
    LDR     R1, [R0]					    	;Ler da Memria o registrador GPIO_PORTM_DEN_R
	MOV     R2, #2_00000001     
    ORR		R1, R2
	STR     R1, [R0]                    		;Escreve no registrador da memria funcionalidade digital
			
; 7. Para habilitar resistor de pull-up interno, setar PUR para 1
	LDR     R0, =GPIO_PORTM_PUR_R				;Carrega o endereo do PUR para a porta M
	MOV     R1, #2_00000001						;Habilitar funcionalidade digital de resistor de pull-up 
	STR     R1, [R0]							;Escreve no registrador da memria do resistor de pull-up

;Interrupes
; 8. Desabilitar a interrupo no registrador IM
	LDR     R0, =GPIO_PORTM_IM_R				;Carrega o endereo do IM para a porta M
	MOV     R1, #2_00							;Desabilitar as interrupes  
	STR     R1, [R0]							;Escreve no registrador
            
; 9. Configurar o tipo de interrupo por borda no registrador IS
	LDR     R0, =GPIO_PORTM_IS_R				;Carrega o endereo do IS para a porta M
	MOV     R1, #2_00							;Por Borda  
	STR     R1, [R0]							;Escreve no registrador

; 10. Configurar  borda nica no registrador IBE
	LDR     R0, =GPIO_PORTM_IBE_R				;Carrega o endereo do IBE para a porta M
	MOV     R1, #2_00							;Borda nica  
	STR     R1, [R0]							;Escreve no registrador

; 11. Configurar  borda de descida (boto pressionado) no registrador IEV
	LDR     R0, =GPIO_PORTM_IEV_R				;Carrega o endereo do IEV para a porta M
	MOV     R1, #2_00							;Borda nica  
	STR     R1, [R0]							;Escreve no registrador

; 12. Habilitar a interrupo no registrador IM
	LDR     R0, =GPIO_PORTM_IM_R				;Carrega o endereo do IM para a porta M
	MOV     R1, #2_01							;Habilitar as interrupes  
	STR     R1, [R0]							;Escreve no registrador
            
;Interrupo nmero 72            
; 13. Setar a prioridade no NVIC
	LDR     R0, =NVIC_PRI18_R           		;Carrega o do NVIC para o grupo que tem o M entre 72 e 75
	MOV     R1, #3			                    ;Prioridade 3							  
	LSL		R1, R1, #5							;Desloca 5 bits para esquerda j que o M  o primeiro Byte de PRI18
	STR     R1, [R0]							;Escreve no registrador da memria

; 14. Habilitar a interrupo no NVIC
	LDR     R0, =NVIC_EN2_R           			;Carrega o do NVIC para o grupo que tem o M entre 64 e 95
	MOV     R1, #1                         
	LSL		R1, #8								;Desloca 8 bits para esquerda j que o M  a interrupo do bit 8 no EN2
	STR     R1, [R0]							;Escreve no registrador da memria

	BX      LR

;################################################################################
; Funo PortN_Output
; Parmetro de entrada: R0 ==> se os BIT1 e o BIT2 esto ligados ou desligados
; Parmetro de sada: No tem
;################################################################################
PortN_Output
	LDR		R1, =GPIO_PORTN_AHB_DATA_R			;Offset do Port
	LDR		R2, [R1]
	BIC		R2, #2_00000011						;Ligando os dois bits lidos da porta R2 = R2 & 11111100
	ORR		R0, R0, R2							;Leitura da Porta OR parmtros passados
	STR		R0, [R1]							;Escreve na Porta N o barramento de dados dos pinos [N5-N0]
	BX 		LR

;################################################################################
; Funo ISR GPIOPortM_Handler (Tratamento da interrupo)
; Parmetro de entrada: No tem
; Parmetro de sada: No tem
;################################################################################
GPIOPortM_Handler
    LDR 	R1, =GPIO_PORTM_ICR_R
    MOV 	R0, #2_00000001 					;Fazendo o ACK do bit 0 do PortM
    STR 	R0, [R1]      						;limpando a interrupo (ack)
    EOR		R10, R10, #2_1
	BX 		LR             						;retorno
 
     
    ALIGN                           			; garante que o fim da seo est alinhada 
    END                             			; fim do arquivo